port_5 = config.ports.port(name="p5", location="10.39.46.4/11/5")[-1]
port_6 = config.ports.port(name="p6", location="10.39.46.4/11/6")[-1]
(device_5, device_6) = config.devices.device(name="device_5").device(name="device_6")
(eth_5,) = device_5.ethernets.ethernet(name="device_5.eth_5")
eth_5.connection.port_name = "p1"
eth_5.mac = "00:00:01:00:00:01"
(eth_6,) = device_6.ethernets.ethernet(name="device_6.eth_6")
eth_6.connection.port_name = "p2"
eth_6.mac = "00:00:02:00:00:01"
(ipv4_5,) = eth_5.ipv4_addresses.ipv4(name="eth_5.ipv4")
ipv4_5.address = "173.173.173.10"
ipv4_5.gateway = "0.0.0.0"
(ipv4_6,) = eth_6.ipv4_addresses.ipv4(name="eth_6.ipv4")
ipv4_6.address = "173.173.173.20"
ipv4_6.gateway = "0.0.0.0"

(app_3, app_4) = config.apps.app(name="app_3").app(name="app_4")
(tcp_3,) = app_3.tcp.tcp(name="tcp1")
tcp_3.keep_alive_time = 120
(tcp_4,) = app_4.tcp.tcp(name="tcp_4")
tcp_4.keep_alive_time = 120


#stateful_2 configs

stateful_2 = app_3.stateful_flows.statefulflow()[-1]
stateful_2.client.protocol = tcp_3.name
stateful_2_c_ep=stateful_2.client.endpoints.client()[-1]
stateful_2_c_ep.ip_interface_name = ipv4_5.name

stateful_2_c_ep.ports.values = [1]

stateful_2.server.protocol = tcp_4.name
stateful_2_sl_ep = stateful_2.server.endpoints.server()[-1]
stateful_2_sl_ep.dest.name = ipv4_6.name
stateful_2_sl_ep.ports.values = [1]
response = api.set_config(config)
response = api.get_config()



import json
import re
import time
from snappi_ixload.timer import Timer


class client_config():
    """
    """
    _HTTP_CLIENT = {
        "enable_tos" : "enableTos",
        # "priority_flow_control_class" : "pfcClass",
        # "precedence_tos" : "precedenceTOS",
        # "delay_tos" : "delayTOS",
        #"throughput_tos"  : "throughputTOS",
        #"reliability_tos"  : "reliabilityTOS",
        "url_stats_count" : "urlStatsCount",
        "disable_priority_flow_control" : "disablePfc",
        "enable_vlan_priority" : "enableVlanPriority",
        "vlan_priority" : "vlanPriority",
        "esm" : "esm",
        "enable_esm" : "enableEsm",
        "time_to_live_value" : "ttlValue",
        #"tcp_close_option" : "tcpCloseOption",
        #"enable_integrity_check_support" : "enableIntegrityCheckSupport",
        "type_of_service" : "tos",
        "high_perf_with_simulated_user" : "highPerfWithSU"       
    } 
    _HTTP_CLIENTS = {
        "version" : "httpVersion",
        "cookie_jar_size": "cookieJarSize",
        "cookie_reject_probability" : "cookieRejectProbability",
        "enable_cookie_support" : "enableCookieSupport",
        "command_timeout" : "commandTimeout",
        "command_timeout_ms" : "commandTimeout_ms",
        "enable_proxy" : "enableHttpProxy",
        #"proxy" : "httpProxy",
        "keep_alive" : "keepAlive",
        "max_sessions" : "maxSessions",
        "max_streams" : "maxStreams",
        "max_pipeline" : "maxPipeline",
        "max_persistent_requests" : "maxPersistentRequests",
        "exact_transactions" : "exactTransactions",
        "follow_http_redirects" : "followHttpRedirects",
        "enable_decompress_support" : "enableDecompressSupport",
        "enable_per_conn_cookie_support" : "enablePerConnCookieSupport",
        #"ip_preference" : "ipPreference",
        "enable_large_header" : "enableLargeHeader",
        "max_header_len" : "maxHeaderLen",
        "per_header_percent_dist" : "perHeaderPercentDist",
        "enable_auth" : "enableAuth",
        "piggy_back_ack" : "piggybackAck",
        "tcp_fast_open" : "tcpFastOpen",
        "content_length_deviation_tolerance" : "contentLengthDeviationTolerance",
        "disable_dns_resolution_cache" : "disableDnsResolutionCache",
        "enable_consecutive_ips_per_session" : "enableConsecutiveIpsPerSession",
        "enable_achieve_cc_first" : "enableAchieveCCFirst",
        "enable_traffic_distribution_for_cc" : "enableTrafficDistributionForCC",
        "browser_emulation_name" : "browserEmulationName"
    }
    _HTTP_GET = {
        "destination": "destination",
        "page"  : "pageObject",
        #"abort" : "abort",
        "profile" : "profile",
        "name_value_args": "namevalueargs",
        "enable_direct_server_return" : "enableDi"
    }

    _HTTP_DELETE = {
        "destination" : "destination",
        "page" : "pageObject",
        "abort" : "abort",
        "profile" : "profile"
    }

    _HTTP_POST = {
        "destination": "destination",
        "page"  : "pageObject",
        "abort" : "abort",
        "profile" : "profile",
        "name_value_args": "namevalueargs",
        "arguments" : "arguments",
        "sending_chunk_size" :  "sendingChunkSize",
        "send_md5_chksum_header": "sendMD5ChkSumHeader"
   }
    _HTTP_PUT = {
        "destination": "destination",
        "page"  : "pageObject",
        "abort" : "abort",
        "profile" : "profile",
        "name_value_args": "namevalueargs",
        "arguments" : "arguments",
        "sending_chunk_size" :  "sendingChunkSize",
        "send_md5_check_sum_header": "sendMD5ChkSumHeader"
   }
    _HTTP_HEADER = {
        "destination": "destination",
        "page"  : "pageObject",
        "abort" : "abort",
        "profile" : "profile",
        "name_value_args": "namevalueargs"
    }

    _TCP = {
        "keep_alive_time": "tcp_keepalive_time"
        
    }
        
    def __init__(self, ixloadapi):
        self._api = ixloadapi
    
    def config(self):
        """
        """
        self._devices_config = self._api._l47config.devices
        with Timer(self._api, "HTTP client Configurations"):
            self._create_client_app()
    
    def _create_client_app(self):
        """Add any scenarios to the api server that do not already exist
        """
        for device in self._devices_config:
            self._create_http_client(device)
            
    def _create_http_client(self, device):
        """Add any scenarios to the api server that do not already exist
        """
        #
        for http in device.https:
            for http_client in http.clients:
            #ip_object = self._ip_list(server.server.name)
            #ip_object = "e2.ipv4"
                url = self._api._config_url.get(http.tcp_name)
                url = self._api.common.get_community_url(url)
                #url = self._api._ixload+"ixload/test/activeTest/communityList/1"
                protocol_url = url+"activityList/"
                options = {}
                options.update({'protocolAndType': "HTTP Client"})
                response = self._api._request('POST', protocol_url, options)
                protocol_url = protocol_url+response
                #self._api._config_url[server.server.name] = protocol_url
                payload = self._api._set_payload(http, client_config._HTTP_CLIENT)
                response = self._api._request('PATCH', protocol_url+"/agent", payload)
                payload = self._api._set_payload(http_client, client_config._HTTP_CLIENTS)
                response = self._api._request('PATCH', protocol_url+"/agent", payload)
                self._create_method(http_client, protocol_url)

        # for client in app_config.http_client:
        #     ip_object = self._api.common.get_ip_name(client.client.name)
        #     url = self._api._config_url.get(self._api._ip_list.get(client.client.name))
        #     url = self._api.common.get_community_url(url)
        #     protocol_url = url+"activityList/"
        #     options = {}
        #     options.update({'protocolAndType': "HTTP Client"})
        #     response = self._api._request('POST', protocol_url, options)
        #     protocol_url = protocol_url+response
        #     self._api._config_url[client.client.name] = protocol_url
        #     payload = self._api._set_payload(client.client, client_config._HTTP_CLIENT)
        #     response = self._api._request('PATCH', protocol_url+"/agent", payload)
        #     self._update_tcp_client(app_config, client)
        #     self._create_method(app_config, client, protocol_url)
        # return
        
    def _update_tcp_client(self, app_config, client):
        #ip_object = self._api.common.get_ip_name(client.client.name)
        for tcp in app_config.tcp:
            #url = self._api._config_url.get(ip_object)
            url = self._api._config_url.get(self._api._ip_list.get(client.client.name))
            url = self._api.common.get_community_url(url)
            tcp_child_url = "%snetwork/globalPlugins" % url
            response_list = self._api._request('GET', tcp_child_url)
            for index in range(len(response_list)):
                if response_list[index]['itemType'] == 'TCPPlugin':
                    tcp_url = "%s/%s" % (tcp_child_url, response_list[index]['objectID'])
                    payload = self._api._set_payload(tcp, client_config._TCP)
                    response = self._api._request('PATCH', tcp_url, payload)

    
    def _create_method(self, http_client, protocol_url):
        for method in http_client.methods:
            for post in method.post:
                payload = self._api._set_payload(post, client_config._HTTP_POST)
                payload.update({'commandType':'POST'})
                command_url = protocol_url+"/agent/actionList"
                response = self._api._request('POST', command_url, payload)
            for get in method.get:
                payload = self._api._set_payload(get, client_config._HTTP_GET)
                payload.update({'commandType':'GET'})
                command_url = protocol_url+"/agent/actionList"
                response = self._api._request('POST', command_url, payload)
            for delete in method.delete:
                payload = self._api._set_payload(delete, client_config._HTTP_DELETE)
                payload.update({'commandType':'DELETE'})
                command_url = protocol_url+"/agent/actionList"
                response = self._api._request('POST', command_url, payload)
            for put in method.put:
                payload = self._api._set_payload(put, client_config._HTTP_PUT)
                payload.update({'commandType':'PUT'})
                command_url = protocol_url+"/agent/actionList"
                response = self._api._request('POST', command_url, payload)
            for header in method.header:
                payload = self._api._set_payload(header, client_config._HTTP_HEADER)
                payload.update({'commandType':'HEAD'})
                command_url = protocol_url+"/agent/actionList"
                response = self._api._request('POST', command_url, payload)







"""
Configure a raw TCP flow with,
- tx port as source to rx port as destination
- frame count 10000, each of size 128 bytes
- transmit rate of 1000 packets per second
Validate,
- frames transmitted and received for configured flow is as expected
"""
import sys
sys.path.append("C:\\Users\\waseebai\\Documents\\GitHub\\snappi\\artifacts\\snappi")
import snappi
# host is IxNetwork API Server
api = snappi.api(location="https://10.39.71.237:443", ext='ixnetwork')
# new config
config = api.config()
# port location is chassis-ip;card-id;port-id
tx, rx = (
    config.ports
    .port(name='p1', location="10.39.64.169;2;1")
    .port(name='p2', location="10.39.64.169;2;2")
)
config.options.port_options.location_preemption = True
# configure layer 1 properties
ly, = config.layer1.layer1(name='ly')
ly.port_names = [tx.name, rx.name]
ly.speed = ly.SPEED_10_GBPS
ly.media = ly.FIBER
# configure flow properties
flw, = config.flows.flow(name='flw')
# flow endpoints
flw.tx_rx.port.tx_name = tx.name
flw.tx_rx.port.rx_name = rx.name
# enable flow metrics
flw.metrics.enable = True
# configure rate, size, frame count
flw.size.fixed = 128
flw.rate.pps = 1000
flw.duration.fixed_packets.packets = 10000
# configure protocol headers with defaults fields
flw.packet.ethernet().vlan().ipv4().tcp()
# push configuration
api.set_config(config)


l47_api = snappi.api(location="http://127.0.0.1:5000", verify=False)

config = l47_api.config()

port_1 = config.ports.port(name="p1", location="10.39.46.4/11/3")[-1]
port_2 = config.ports.port(name="p2", location="10.39.46.4/11/7")[-1]

(d1, d2) = config.devices.device(name="d1").device(name="d2")
(e1,) = d1.ethernets.ethernet(name="d1.e1")
e1.connection.port_name = "p1"
e1.mac = "00:00:01:00:00:01"
(e2,) = d2.ethernets.ethernet(name="d2.e2")
e2.connection.port_name = "p2"
e2.mac = "00:00:02:00:00:01"
(ip1,) = e1.ipv4_addresses.ipv4(name="e1.ipv4")
ip1.address = "173.173.173.10"
ip1.gateway = "0.0.0.0"
(ip2,) = e2.ipv4_addresses.ipv4(name="e2.ipv4")
ip2.address = "173.173.173.20"
ip2.gateway = "0.0.0.0"

(a1, a2) = config.apps.app(name="a1").app(name="a2")
(t1,) = a1.tcp.tcp(name="tcp1")
t1.keep_alive_time = 120
(t2,) = a2.tcp.tcp(name="tcp2")
t2.keep_alive_time = 120

(h1,) = a1.http_client.httpclient()
h1.client.name="HTTP_client1"
h1.client.transport_name = t1.name
h1.client.command_timeout = 60
h1.client.cookie_reject_probability = False
(m1,) = h1.client.methods.get()
m1.server = "Traffic2_HTTPServer1:80"
m1.page = "/64k.html"


(h2,) = a2.http_server.httpserver()
#h2.server.name="HTTP_server1"
h2.server.name = "HTTP_server1"
h2.server.transport_name = t2.name
h2.server.http_port = 80

#Traffic configs

traffic = a1.stateful_flows.statefulflow()[-1]
traffic.client.protocol = h1.client.name
traffic_c_ep=traffic.client.endpoints.client()[-1]
traffic_c_ep.ip_interface_name = ip1.name

traffic_c_ep.ports.values = [1]

traffic.server.protocol = h2.server.name  
traffic_sl_ep = traffic.server.endpoints.server()[-1]
traffic_sl_ep.dest.name = ip2.name
traffic_sl_ep.ports.values = [1]
response = l47_api.set_config(config)
print(response)

#import pdb;pdb.set_trace()
response = l47_api.get_config()
print(response)




from snappi_ixload.logger import get_ixload_logger

class Common(object):
    def __init__(self):
        self.logger = get_ixload_logger(__name__)

    def get_ip_name(self, name):
        '''
            return the IP config from statefull flow endpoints.
        '''
        for app in self._api._apps:
            for flow in app.stateful_flows:
                if flow.client.protocol == name:
                    for endpoint in flow.client.endpoints:
                        return "e1.ipv4"
                elif flow.server.protocol == name:
                    for endpoint in flow.server.endpoints:
                        return "e2.ipv4"
    
    def get_protocol_ip(self, apps_obj):
        return_list = []
        for app in apps_obj.apps:
            for flow in app.stateful_flows:
                if flow.client.endpoints and flow.server.endpoints:
                    temp_dict, client, server = {}, {}, {}
                    for endpoint in flow.client.endpoints:
                        temp_dict[flow.client.protocol] = endpoint.ip_interface_name
                    for endpoint in flow.server.endpoints:
                        temp_dict[flow.server.protocol] = endpoint.dest.name               
                    #temp_dict['client'] = client
                    #temp_dict['server'] = server
                    #return_list.append(temp_dict)             
        return temp_dict

    
    def get_community_url(self, url):
        import re
        match = re.match(r'(.*\/communityList\/)(\d+)', url)
        if match:
            return (match.group(1)+match.group(2)+"/")


api = snappi.api(location="http://127.0.0.1:5000", verify=False)

config = api.config()

port_1 = config.ports.port(name="p1", location="10.39.46.4/11/3")[-1]
port_2 = config.ports.port(name="p2", location="10.39.46.4/11/7")[-1]
(device_3, device_4) = config.devices.device(name="device_3").device(name="device_4")
(eth_3,) = device_3.ethernets.ethernet(name="device_3.eth_3")
eth_3.connection.port_name = "p1"
eth_3.mac = "00:00:01:00:00:01"
(eth_4,) = device_4.ethernets.ethernet(name="device_4.eth_4")
eth_4.connection.port_name = "p2"
eth_4.mac = "00:00:02:00:00:01"
(ipv4_3,) = eth_3.ipv4_addresses.ipv4(name="eth_3.ipv4")
ipv4_3.address = "173.173.173.10"
ipv4_3.gateway = "0.0.0.0"
(ipv4_4,) = eth_4.ipv4_addresses.ipv4(name="eth_4.ipv4")
ipv4_4.address = "173.173.173.20"
ipv4_4.gateway = "0.0.0.0"

(app_1, app_2) = config.apps.app(name="app_1").app(name="app_2")
(tcp_1,) = app_1.tcp.tcp(name="tcp1")
tcp_1.keep_alive_time = 120
(tcp2,) = app_2.tcp.tcp(name="tcp2")
tcp2.keep_alive_time = 120

(http_client_1,) = app_1.http_client.httpclient()
http_client_1.client.name="HTTP_client1"
http_client_1.client.transport_name = tcp_1.name
http_client_1.client.command_timeout = 60
http_client_1.client.cookie_reject_probability = False
(m1,) = http_client_1.client.methods.get()
m1.server = "Traffic2_HTTPServer1:80"
m1.page = "/64k.html"


(http_client_2,) = app_2.http_server.httpserver()
#http_client_2.server.name="HTTP_server1"
http_client_2.server.name = "HTTP_server1"
http_client_2.server.transport_name = tcp2.name
http_client_2.server.http_port = 80

#Traffic configs

traffic = app_1.stateful_flows.statefulflow()[-1]
traffic.client.protocol = http_client_1.client.name
traffic_c_ep=traffic.client.endpoints.client()[-1]
traffic_c_ep.ip_interface_name = ipv4_3.name

traffic_c_ep.ports.values = [1]

traffic.server.protocol = http_client_2.server.name  
traffic_sl_ep = traffic.server.endpoints.server()[-1]
traffic_sl_ep.dest.name = ipv4_4.name
traffic_sl_ep.ports.values = [1]

#Testcase for tcp as endpoints

port_5 = config.ports.port(name="p5", location="10.39.46.4/11/5")[-1]
port_6 = config.ports.port(name="p6", location="10.39.46.4/11/6")[-1]
(device_5, device_6) = config.devices.device(name="device_5").device(name="device_6")
(eth_5,) = device_5.ethernets.ethernet(name="device_5.eth_5")
eth_5.connection.port_name = "p5"
eth_5.mac = "00:00:01:00:00:01"
(eth_6,) = device_6.ethernets.ethernet(name="device_6.eth_6")
eth_6.connection.port_name = "p6"
eth_6.mac = "00:00:02:00:00:01"
(ipv4_5,) = eth_5.ipv4_addresses.ipv4(name="eth_5.ipv4")
ipv4_5.address = "173.173.173.10"
ipv4_5.gateway = "0.0.0.0"
(ipv4_6,) = eth_6.ipv4_addresses.ipv4(name="eth_6.ipv4")
ipv4_6.address = "173.173.173.20"
ipv4_6.gateway = "0.0.0.0"

(app_3, app_4) = config.apps.app(name="app_3").app(name="app_4")
(tcp_3,) = app_3.tcp.tcp(name="tcp_3")
tcp_3.keep_alive_time = 120
(tcp_4,) = app_4.tcp.tcp(name="tcp_4")
tcp_4.keep_alive_time = 120


#stateful_2 configs

stateful_2 = app_3.stateful_flows.statefulflow()[-1]
stateful_2.client.protocol = tcp_3.name
stateful_2_c_ep=stateful_2.client.endpoints.client()[-1]
stateful_2_c_ep.ip_interface_name = ipv4_5.name

stateful_2_c_ep.ports.values = [1]

stateful_2.server.protocol = tcp_4.name
stateful_2_sl_ep = stateful_2.server.endpoints.server()[-1]
stateful_2_sl_ep.dest.name = ipv4_6.name
stateful_2_sl_ep.ports.values = [1]

# Testcase for One arm test

port_1 = config.ports.port(name="p7", location="10.39.46.4/11/7")[-1]
(device_7,)= config.devices.device(name="device_7")
(eth_7,) = device_7.ethernets.ethernet(name="device_7.eth_7")
eth_7.connection.port_name = "p7"
eth_7.mac = "00:00:01:00:00:01"
(ipv4_7,) = eth_7.ipv4_addresses.ipv4(name="eth_7.ipv4")
ipv4_7.address = "173.173.173.10"
ipv4_7.gateway = "0.0.0.0"

(app_5,)  = config.apps.app(name="app_5")
(tcp_5,) = app_5.tcp.tcp(name="tcp_5")
tcp_5.keep_alive_time = 120

(http_client_1,) = app_5.http_client.httpclient()
http_client_1.client.name="HTTP_client1"
http_client_1.client.transport_name = tcp_5.name
http_client_1.client.command_timeout = 60
http_client_1.client.cookie_reject_probability = False
(m1,) = http_client_1.client.methods.get()
m1.server = "10.10.10.100"
m1.page = "/64k.html"

#stateful_3 configs
traffic = app_5.stateful_flows.statefulflow()[-1]
traffic.client.protocol = http_client_1.client.name
traffic_c_ep=traffic.client.endpoints.client()[-1]
traffic_c_ep.ip_interface_name = ipv4_7.name
traffic_c_ep.ports.values = [1]

traffic_sl_ep = traffic.server.endpoints.server()[-1]
traffic_sl_ep.dest.ipv4_address = "10.10.10.100"

response = api.set_config(config)
print(response)

response = api.get_config()
print(response)


import json
import re
from brisk_ixload.timer import Timer


class protocols(object):
    """Transforms OpenAPI objects into IxLoad objects
    - network to /network
    Args
    ----
    - ixloadapi (Api): instance of the Api class
    """
    
    def __init__(self, ixloadapi):
        self._api = ixloadapi
    
    def config(self):
        """Transform config.network into IxLoad.network
        1) create a network for every config.scenarios.networks that is not present in IxLoad
        2) set config.networks[].name to /network -name using rest call
        """
        self._scenarios_config = self._api.brisk_config.scenarios
        with Timer(self._api, "Protocol Configuration"):
            self._create_protocol(self._scenarios_config)
    
    def _create_protocol(self, scenarios):
        """Add any network to the api server that do not already exist
        """
        for scenario in scenarios:
            for network in scenario.networks:
                if network.protocols:
                    for protocol in network.protocols:
                        if protocol.httpclient:
                            url1, url2 = self._api._get_url(scenario.url , protocol.url)
                            payload = {"protocolAndType":"HTTP Client"}
                            response = self._api._request('POST', url1, payload)
                            protocol.url = url1 + response + url2
                            for agent in protocol.httpclient:
                                self._configure_protocol(agent, protocol.url)
                                for cmd in agent.commands:        
                                    self._configure_command(cmd, agent.url)
                        if protocol.httpserver:
                            url1, url2 = self._api._get_url(scenario.url , protocol.url)
                            payload = {"protocolAndType":"HTTP Server"}
                            response = self._api._request('POST', url1, payload)
                            protocol.url = url1 + response + url2
                            for agent in protocol.httpserver:
                                #import pdb; pdb.set_trace()
                                self._configure_protocol(agent, protocol.url)
                                #for cmd in agent.commands:        
                                    #self._configure_command(cmd, agent.url)

    def _configure_protocol(self, agent, protocol_url):
        """Add any network to the api server that do not already exist
        """
        agent.url = protocol_url + agent.url
        payload = self._api._set_payload(agent)
        if payload:
            #import pdb; pdb.set_trace()
            self._api._request('PATCH', agent.url, payload)
        return

    def _configure_command(self, cmd, agent_url):
        """Add any network to the api server that do not already exist
        """
        cmd.url = agent_url + cmd.url
        payload = self._api._set_payload(cmd)
        #import pdb; pdb.set_trace()
        if payload:
            #import pdb; pdb.set_trace()
            self._api._request('POST', cmd.url, payload)
        return




import json
import re
import time
from snappi_ixload.timer import Timer

class server_config():
    """
    """
    _HTTP_SERVER = {
        "enable_tos" : "enableTos",
        #"priority_flow_control_class" : "pfcClass",
        #"precedence_tos" : "precedenceTOS",
        #"delay_tos" : "delayTOS",
        #"throughput_tos"  : "throughputTOS",
        "url_stats_count" : "urlStatsCount",
        "disable_priority_flow_control" : "disablePfc",
        "enable_vlan_priority" : "enableVlanPriority",
        "vlan_priority" : "vlanPriority",
        "esm" : "esm",
        "enable_esm" : "enableEsm",
        "time_to_live_value" : "ttlValue",
        #"tcp_close_option" : "tcpCloseOption",
        "type_of_service" : "tos",
        "high_perf_with_simulated_user" : "highPerfWithSU"
    }

    _HTTP_SERVERS = {
        "rst_timeout" : "rstTimeout",
        "enable_http2" : "enableHTTP2",
        "port" : "httpPort",
        "request_timeout" : "requestTimeout",
        "maximum_response_delay" : "maxResponseDelay",
        "minimum_response_delay" : "minResponseDelay",
        "dont_expect_upgrade" : "dontExpectUpgrade",
        "enable_per_server_per_url_stat" : "enablePerServerPerURLstat",
        "url_page_size" : "urlPageSize",
        "enable_chunk_encoding" : "enableChunkEncoding",
        #"integrity_check_option" : "integrityCheckOption",
        "enable_md5_checksum" : "enableMD5Checksum",
    }

    _TCP = {
        "keep_alive_time": "tcp_keepalive_time"
    }
        
    def __init__(self, ixloadapi):
        self._api = ixloadapi
    
    def config(self):
        """
        """
        self._devices_config = self._api._l47config.devices
        with Timer(self._api, "HTTP server Configurations"):
            self._create_server_app()
    
    def _create_server_app(self):
        """Add any scenarios to the api server that do not already exist
        """
        for device in self._devices_config:
            self._create_http_server(device)
            
    def _create_http_server(self, device):
        """Add any scenarios to the api server that do not already exist
        """
        for http in device.https:
            for http_server in http.servers:
            #ip_object = self._ip_list(server.server.name)
            #ip_object = "e2.ipv4"
                url = self._api._config_url.get(http.tcp_name)
                url = self._api.common.get_community_url(url)
                #url = self._api._ixload+"ixload/test/activeTest/communityList/1"
                protocol_url = url+"activityList/"
                options = {}
                options.update({'protocolAndType': "HTTP Server"})
                response = self._api._request('POST', protocol_url, options)
                protocol_url = protocol_url+response
                #self._api._config_url[server.server.name] = protocol_url
                payload = self._api._set_payload(http, server_config._HTTP_SERVER)
                response = self._api._request('PATCH', protocol_url+"/agent", payload)
                payload = self._api._set_payload(http_server, server_config._HTTP_SERVERS)
                response = self._api._request('PATCH', protocol_url+"/agent", payload)
            del(http)
            #self._update_tcp_server(app_config, server)
    
    def _update_tcp_server(self, app_config, server):
        #ip_object = self._api.common.get_ip_name(self._server_config, server.server.name)
        for tcp in app_config.tcp:
            url = self._api._config_url.get(self._api._ip_list.get(server.server.name))
            #url = self._api._config_url.get(ip_object)
            url = self._api.common.get_community_url(url)
            tcp_child_url = "%snetwork/globalPlugins" % url
            response_list = self._api._request('GET', tcp_child_url)
            for index in range(len(response_list)):
                if response_list[index]['itemType'] == 'TCPPlugin':
                    tcp_url = "%s/%s" % (tcp_child_url, response_list[index]['objectID'])
                    payload = self._api._set_payload(tcp, server_config._TCP)
                    response = self._api._request('PATCH', tcp_url, payload)





import sys
#sys.path.append("C:\\Users\\waseebai\\Documents\\GitHub\\snappi\\artifacts\\snappi")
#sys.path.insert(0, "C:\\Users\\waseebai\\Documents\\project\\GitHub\\snappi\\artifacts\\snappi")


import snappi

api = snappi.api(location="http://127.0.0.1:5000", verify=False)
config = api.config()

port_1 = config.ports.port(name="p1", location="amit.buh.is.keysight.com/1/1")[-1]
port_2 = config.ports.port(name="p2", location="amit.buh.is.keysight.com/1/2")[-1]

(d1, d2) = config.devices.device(name="d1").device(name="d2")
(e1,) = d1.ethernets.ethernet(name="d1.e1")
#(e1,e3) = d1.ethernets.ethernet(name="d1.e1").ethernet(name="d1.e3")
e1.connection.port_name = "p1"
e1.mac = "70:9C:91:69:00:00"
e1.step = "00:00:00:00:00:01" 
e1.count = 1
#e3.mac = "00:00:01:00:00:03"
#e3.step = "00:00:00:00:00:01" 
#e3.count = 1

(e2,) = d2.ethernets.ethernet(name="d2.e2")
e2.connection.port_name = "p2"
e2.mac = "7C:5D:68:26:00:00"

(vlan1,) = e1.vlans.vlan(name = "vlan1")
vlan1.id = 1
vlan1.priority = 1
vlan1.tpid = "x8100"
#(vlan3,) = e3.vlans.vlan(name = "vlan3")
#vlan3.id = 1
#vlan3.priority = 1
#vlan3.tpid = "x8100"
(vlan2,) = e2.vlans.vlan(name = "vlan2")
vlan2.id = 1
vlan2.priority = 1
vlan2.tpid = "x8100"
(ip1,) = e1.ipv4_addresses.ipv4(name="e1.ipv4")
ip1.address = "173.173.173.10"
ip1.gateway = "0.0.0.0"
ip1.step = "0.0.0.1"
ip1.count = 1
#(ip3,) = e3.ipv4_addresses.ipv4(name="e3.ipv4")
#ip3.address = "173.173.173.20"
#ip3.gateway = "0.0.0.0"
#ip3.step = "0.0.0.1"
#ip3.count = 1
(ip2,) = e2.ipv4_addresses.ipv4(name="e2.ipv4")
ip2.address = "173.173.173.30"
ip2.gateway = "0.0.0.0"



#TCP/UDP configs

(t1,) = d1.tcps.tcp(name="Tcp1")
t1.ip_interface_name = ip1.name
t1.adjust_tcp_buffers = False
t1.keep_alive_time = 7000
t1.keep_alive_interval = 60
t1.receive_buffer_size = 8065
t1.transmit_buffer_size = 8065
t1.adjust_tcp_buffers = True
t1.retransmission_minimum_timeout = 180
t1.retransmission_maximum_timeout = 1000
t1.minimum_source_port = 80
t1.maximum_source_port = 1024
t1.inter_packet_gap = 9
t1.inter_packet_delay = 1
t1.ip_fragment_time = 1
t1.fin_timeout = 60
t1.syn_retries = 5
t1.synack_retries = 5
t1.retansmit_retries1 = 3
t1.retransmit_retries2 = 3
t1.packet_reordering = 3
t1.delayed_acks_segments = 0
t1.delayed_acks_timeout = 0
t1.port_randomization = False
t1.disable_path_mtu = False
t1.window_scaling = False
t1.selective_ack = False
t1.time_wait_reuse = False
t1.time_wait_recycle = False
t1.time_wait_rfc1323_strict = False
t1.packet_timestamps = False
t1.explicit_congestion_notification  = False
t1.bic = 0
t1.vegas_alpha = 2
t1.rfc1337 = True
t1.mem_low = 24756
t1.maximum_wmem = 262144
t1.westwood = 0
t1.avoid_vegas_congestion = 0
t1.maximum_rmem = 262144
t1.orphan_retries = 0
t1.maximum_time_wait_buckets = 1800
t1.low_latency = 0
t1.minimum_rmem = 4096
t1.window_scale = 2
t1.wmem_default = 262144
t1.minimum_wmem = 4096
t1.stdurg = False
t1.maximum_syn_backlog = 1024
t1.dsack = True
t1.abort_on_overflow = True
t1.fragment_reassembly_timer = 0
t1.vegas_beta = 6
t1.maximum_orphans = 8192
t1.mem_pressure = 32768
t1.moderate_receive_buffer = 0
t1.no_metrics_save = True
t1.retrans_collapse = True
t1.rmem_default = 262144
t1.mem_high = 49152
t1.vegas_gamma = 2
t1.fack = True
t1.bic_low_window = 14
t1.app_win = 31
t1.keep_alive_probes = 9






# t1.rmem_default = 65536
# t1.transmit_buffer_size = 65536
# t1.time_wait_recycle = False
# t1.time_wait_rfc1323_strict = True
# t1.keep_alive_time = 600

(t2,) = d2.tcps.tcp(name="Tcp2")
t2.ip_interface_name = ip2.name
t2.time_wait_recycle = False
t2.time_wait_rfc1323_strict = True
t2.keep_alive_time = 600
(http_1,) = d1.https.http(name="HTTP1")
http_1.tcp_name = t1.name   #UDP configs can be mapped http.transport = udp_1.name
http_1.enable_tos = False
http_1.priority_flow_control_class = "v10"
http_1.precedence_tos = "v20"
http_1.delay_tos = "v10"
http_1.throughput_tos = "v10"
http_1.url_stats_count = 10
http_1.disable_priority_flow_control =  0
http_1.enable_vlan_priority = False
http_1.vlan_priority = 0
http_1.esm = 1460
http_1.enable_esm =  False
http_1.time_to_live_value = 64
http_1.tcp_close_option = "v10"
http_1.enable_integrity_check_support = False
http_1.type_of_service = 0
http_1.high_perf_with_simulated_user =  False
(http_client,) = http_1.clients.client()
http_client.cookie_jar_size = 100
http_client.version = "1"
http_client.cookie_reject_probability = True
http_client.enable_cookie_support = False
http_client.command_timeout = 600
http_client.command_timeout_ms = 0
http_client.enable_proxy = False
http_client.keep_alive = False
http_client.max_sessions = 3
http_client.max_streams = 1
http_client.max_pipeline = 1
http_client.max_persistent_requests = 1
http_client.exact_transactions = 0
http_client.follow_http_redirects = False
http_client.enable_decompress_support = False
http_client.enable_per_conn_cookie_support = False
http_client.ip_preference = "v10"
http_client.enable_large_header = False
http_client.max_header_len = 1024
http_client.per_header_percent_dist =  False
http_client.enable_auth = False
http_client.piggy_back_ack = True
http_client.tcp_fast_open = False
http_client.content_length_deviation_tolerance = 0
http_client.disable_dns_resolution_cache = False
http_client.enable_consecutive_ips_per_session = False
http_client.enable_achieve_cc_first = False
http_client.enable_traffic_distribution_for_cc = False
http_client.browser_emulation_name = "Browser1"

#http_1.client(endpoints_allow_inbound)

# get1 = http1.client.methods.add("get")
# get1.page = "./1b.html"
# get1.destination = "10.0.10.1" #real http server ip or emulated http object  get1.destination = "http2:80"
#for http server emulation
#get1.destination = http_2.name 
(http_2,) = d2.https.http(name="HTTP2")
http_2.tcp_name = t2.name		#UDP configs can be mapped http.transport = udp_2.name
http_2.enable_tos = False
http_2.priority_flow_control_class = "v10"
http_2.precedence_tos = "v20"
http_2.delay_tos = "v10"
http_2.throughput_tos = "v10"
http_2.url_stats_count = 10
http_2.disable_priority_flow_control = 0
http_2.enable_vlan_priority = False
http_2.vlan_priority = 0
http_2.esm = 1460
http_2.enable_esm = False 
http_2.time_to_live_value = 64
http_2.tcp_close_option = "v10"
http_2.enable_integrity_check_support = False
http_2.type_of_service = 0 
http_2.high_perf_with_simulated_user = False		#UDP configs can be mapped http.transport = udp_2.name
#http_2.server(endpoints_allow_outbound)
(http_server,) = http_2.servers.server()
http_server.rst_timeout = 100
http_server.enable_http2 = False
http_server.port = 80
http_server.request_timeout = 300
http_server.maximum_response_delay = 0
http_server.minimum_response_delay = 0
http_server.dont_expect_upgrade = False
http_server.enable_per_server_per_url_stat = False
http_server.url_page_size = 1024
http_server.enable_chunk_encoding = False
http_server.enable_md5_checksum = False

(get_a,delete_a) = http_client.methods.method().method()
(get1,) = get_a.get.get()
get1.destination = "Traffic2_HTTPServer1:80" 
get1.page = "./1b.html"
#get1.destination = "Traffic2_HTTPServer1:80" #real http server ip or emulated http object  get1.destination = "http2:80"
#for http server emulation
#get1.destination = http_2.name   
# (post1,) = post1_a.post.post()
# post1.destination = "Traffic2_HTTPServer1:80" 
# post1.page = "./1b.html"
(delete1,) = delete_a.delete.delete()
delete1.destination = "Traffic2_HTTPServer1:80" 
delete1.page = "./1b.html"



response = api.set_config(config)
print(response)

cs = api.control_state()
cs.app.state = 'start' #cs.app.state.START 
response1 = api.set_control_state(cs) 
print(response1)
cs.app.state = 'stop' #cs.app.state.START 
api.set_control_state(cs)            







import json


class Validation(object):
    """Validate the configuration

    Ensures entire configuration has unique names

    Args
    ----
    - ixnetworkapi (Api): instance of the ixnetworkapi class
    """
    def __init__(self, ixnetworkapi):
        self._api = ixnetworkapi
    
    def validate_config(self):
        self._unique_name_errors = []
        self.__check_config_objects(self._api.snappi_config)
        if len(self._unique_name_errors) > 0:
            raise NameError(', '.join(self._unique_name_errors))

    def __check_config_objects(self, config_item):
        if config_item is None:
            return

        if hasattr(config_item, 'choice') is True and getattr(
                config_item, 'choice') is None:
            return
            
        for attr_name in config_item._properties:
            if attr_name.startswith('_') or \
                        attr_name == 'parent':
                continue
            attr_value = getattr(config_item, attr_name, None)
            if callable(attr_value) is True:
                continue
            
            if attr_name == 'name':
                if attr_value in self._api._config_objects:
                    self._unique_name_errors.append('%s.name: "%s" is not unique' % (config_item.__class__.__name__, attr_value))
                # todo: we will enable after finalyze model
                if attr_value is None:
                    continue
                    # self._unique_name_errors.append('%s.name: "None" is not allowed' % (config_item.__class__.__name__))
                else:
                    self._api._config_objects[attr_value] = config_item
            elif '__module__' in dir(attr_value):
                if attr_value.__module__.startswith('snappi'):
                    if '__next__' in dir(attr_value):
                        for item in attr_value:
                            if getattr(item, 'parent', None) is not None:
                                self.__check_config_objects(item.parent)
                            else:
                                self.__check_config_objects(item)
                    else:
                        self.__check_config_objects(attr_value)

     


#import errors as err
import sys
import traceback
#sys.path.insert(0, "c:\\Users\\waseebai\\Documents\\project\\snappi_l47\\snappi\\artifacts\\snappi")
import snappi

class Snappil47Exception(Exception): 
    def __init__(self, *args):
        super(Snappil47Exception, self).__init__(*args)
        self._args = args
        self._message = None
        self._status_code = None
        self._url = None
        self.process_exception()

    @property
    def args(self):
        return (
            self._status_code,
            (
                [self._message]
                if not isinstance(self._message, list)
                else self._message
            ),
            self._url
        )

    @property
    def message(self):
        return self._message.args[0]
    
    @property
    def url(self):
        return self._url

    @property
    def status_code(self):
        return self._status_code
    
    def process_exception(self):
        #import pdb;pdb.set_trace()
        if isinstance(self._args[0].args, tuple) and len(self._args[0].args) == 1:
            if "Max retries exceeded" in str(self._args[0].args[0]):
                self._status_code= 500
                self._message = self._args[0]
                return (self._message, self._status_code)
            if isinstance(self._args[0].args[0], (str, list)):
                self._status_code = (
                    400 if self._status_code is None else self._status_code
                )
                self._message = self._args[0].args[0]
                return (self._message, self._status_code) 
            if isinstance(self._args[0], (NameError, TypeError, ValueError, KeyError, AttributeError)):
                self._status_code= 400
                self._message= self._args[0].args
                return (self._status_code, self._message)
            if isinstance(
                self._args[0], (ImportError, RuntimeError)
            ):
                self._status_code= 400
                self._message = self._args[0]
                return (self._status_code, self._message)
            else:
                self._status_code= 400
                self._message = self._args
                return (self._status_code, self._message)
        else:
            self._message = self._args
        return


import ixloadapi
import brisk
cObj = brisk.Api().config()
scObj = cObj.scenarios.scenario().scenario().scenario()
nwObj = scObj[-1].networks.network()
etObj = nwObj[-1].ethernet.ethernet()
ipv4Obj = nwObj[-1].ipv4.ipv4()

api = ixloadapi.Api("10.22.45.55", ixload_version = "8.50.0.465")
api.set_config(cObj)
print("*****")
print(cObj)
print("*****")



import json
import time, re
import sys
import logging
#import ixrestutils as http_transport
from collections import namedtuple
#import sys
#sys.path.append("C:\\Users\\waseebai\\Documents\\GitHub\\snappi\\artifacts\\snappi")
#sys.path.insert(0, "C:\\Users\\waseebai\\Documents\\project\\GitHub\\snappi\\artifacts\\snappi")
#sys.path.insert(0, "c:\\Users\\waseebai\\Documents\\project\\snappi_l47\\snappi\\artifacts\\snappi")
import snappi
import snappi_ixload.ixrestutils as http_transport
from snappi_ixload.interface import interfaces
from snappi_ixload.http_config import client_config 
from snappi_ixload.tcp import tcp_config 
from snappi_ixload.http_server_config import server_config
from snappi_ixload.logger import setup_ixload_logger 
from snappi_ixload.common import Common
from snappi_ixload.exceptions import Snappil47Exception
from snappi_ixload.ports import port
#from protocols import protocols
#from snappi_ixload.chassis import chassis
#from stats import stats


class Api(snappi.Api):
    """
    """
    def __init__(self,
                 host,
                 version, 
                 username='admin',
                 password='admin',
                 gateway_port = 8080,
                 log_level='info',
                 **kwargs):
        """Create a session
        - address (str): The ip address of the TestPlatform to connect to
        where test sessions will be created or connected to.
        - port (str): The rest port of the TestPlatform to connect to.
        - username (str): The username to be used for authentication
        - password (str): The password to be used for authentication
        """
        super().__init__(**kwargs)
        #    host='https://127.0.0.1:11009' if host is None else host
        #)
        self._host = host
        self._address, self._port = self._get_addr_port(self._host)
        self._username = username
        self._password = password
        self._config_url = {}
        
        self.ixload_version = version
        self._gateway_port = gateway_port
        self._l4config = None
        self._assistant = None
        self._config_type = self.config()
        self.interfaces = interfaces(self)
        self.tcp = tcp_config(self)
        self.common = Common()
        self.http_cl = client_config(self)
        self.http_sr = server_config(self)
        #import pdb;pdb.set_trace()
        self.port = port(self)
        self._log_level = (
            logging.INFO
            if kwargs.get("loglevel") is None
            else kwargs.get("loglevel")
        )
        self.logger = setup_ixload_logger(self.log_level, module_name=__name__)
        #self.protocols = protocols(self)
        #self.chassis = chassis(self)
        #self.stats = stats(self)
    
    def _get_addr_port(self, host):
        items = host.split('/')
        items = items[-1].split(':')

        addr = items[0]
        if len(items) == 2:
            return addr, items[-1]
        else:
            if host.startswith('https'):
                return addr, '443'
            else:
                return addr, '80'
    @property
    def _config(self):
        return self._config
    
    @property
    def log_level(self):
        return self._log_level
        
    def _request_detail(self):
        request_detail = snappi.Warning()
        errors = self._errors
        warnings = list()
        if errors:
            Snappil47Exception(errors)
        warnings.append("")
        request_detail.warnings = warnings
        return request_detail
    
    def set_config(self, config):
        """Set or update the configuration
        """
        try:
            if isinstance(config, (type(self._config_type),
                                    str)) is False:
                raise TypeError(
                    'The content must be of type Union[Config, str]')

            if isinstance(config, str) is True:
                config = self._config_type.deserialize(config)
            self._config_objects = {}
            self._device_encap = {}
            self._ixl_objects = {}
            self._connect()
            
            #self._ip_list = self.common.get_protocol_ip(config)
            self._l47config = config
            self.interfaces.config()
            self.tcp.config()
            self.http_sr.config()
            self.http_cl.config()
            self.port.config()
            self._apply_config()    
        except Exception as err:
            self.logger.info(f"error:{err}")
            raise Snappil47Exception(err)
        return self._request_detail()
    
    def add_error(self, error):
        """Add an error to the global errors"""
        if isinstance(error, str) is False:
            self._errors.append("%s %s" % (type(error), str(error)))
        else:
            self._errors.append(error)

    def get_config(self):
        return self._l47config

    def set_control_state(self, config):
        try:
            if config.app.state == "start":
                url = "%sixload/test/operations/runTest" % (self._ixload)
                payload = {}
                
                reply = self._request('POST', url, payload, option=1)
                if not reply.ok:
                    raise Exception(reply.text)
                #self._wait_for_action_to_finish(reply, url)
                msg = (
                        "Traffic are in running state. "
                        "Please stop those using set_control_state"
                    )
                #self.add_error(msg)
                self.warning(msg)
                
            elif config.app.state == "stop":
                url = "%s/ixload/test/operations/gracefulStopRun" % (self._ixload)
                payload = {}
                reply = self._request('POST', url, payload, option=1)
                self._wait_for_action_to_finish(reply, url)
            elif config.app.state == "abort":
                url = "%s/ixload/test/operations/abortAndReleaseConfigWaitFinish" % (self._ixload)
                payload = {}
                reply = self._request('POST', url, payload, option=1)
                self._wait_for_action_to_finish(reply, url)
        except Exception as err:
            self.logger.info(f"error:{err}")
            raise Snappil47Exception(err)
        return self._request_detail()
        
    def stats(self, config):
        """Set or update the configuration
        """
        try:
            if isinstance(config, (type(self._config_type),
                                    str)) is False:
                raise TypeError(
                    'The content must be of type Union[Config, str]')

            if isinstance(config, str) is True:
                config = self._config_type.deserialize(config)
            self._config = config
        except Exception as err:
            print(err)
    
    def _apply_config(self):
        """
            Apply configs
        """
        
        #url = "%s/ixload/test/operations/saveAs" % (self._ixload)
        #payload = {"fullPath":"c:/Users/waseebai/Documents/project/test_snappi_l471.rxf", "overWrite": 1}
        #reply = self._request('POST', url, payload, option=1)
        #self._wait_for_action_to_finish(reply, url)
        url = self._ixload + "ixload/test/operations/applyConfiguration"
        payload = {}
        reply = self._request('POST', url, payload, option=1)
        if not reply.ok:
            raise Exception(reply.text)
        self.logger.info("Cofiguration applied :%s" % (reply))
        self._wait_for_action_to_finish(reply, url)

    def run_test(self):
        """
            start test
        """
        url = "%s/ixload/test/operations/applyConfiguration" % (self._ixload)
        payload = {}
        reply = self._request('POST', url, payload, option=1)
        self._wait_for_action_to_finish(reply, url)
        url = "%s/ixload/test/operations/runTest" % (self._ixload)
        payload = {}
        reply = self._request('POST', url, payload, option=1)
        self._wait_for_action_to_finish(reply, url)
        return (self.get_current_state())
    
    def get_current_state(self):
        '''
        This method gets the test current state. (for example - running, unconfigured, ..)
        '''
        url = "%s/ixload/test/activeTest" % (self._ixload)
        reply = self._request('GET', url, option=1)
        return reply['currentState']
    
    def strip_api_and_version_from_url(self, url):
        '''
        #remove the slash (if any) at the beginning of the url
        '''
        if url[0] == '/':
            url = url[1:]
        url_elements = url.split('/')
        if 'api' in url:
            #strip the api/v0 part of the url
            url_elements = url_elements[2:]
        return '/'.join(url_elements)
            
    def sessionAssistance(self):
        '''
        '''
        self.connection = http_transport.get_connection(self._host, self._gateway_port, http_redirect=True)
        session_url = "sessions"
        data = {"ixLoadVersion": self.ixload_version}
        data = json.dumps(data)
        reply = self.connection.http_post(url=session_url, data=data)
        if not reply.ok:
            raise Exception(reply.text)
        try:
            new_obj_path = reply.headers['location']
        except:
            raise Exception("Location header is not present. Please check if the action was created successfully.")
        session_id = new_obj_path.split('/')[-1]
        self._ixload = "%s/%s" % (session_url, session_id)
        start_session_url = "%s/operations/start" % (self._ixload)
        reply = self.connection.http_post(url=start_session_url, data={})
        if not reply.ok:
            Snappil47Exception.status_code = reply.status_code
            raise Exception(reply.text)
        action_result_url = reply.headers.get('location')
        if action_result_url:
            action_result_url = self.strip_api_and_version_from_url(action_result_url)
            action_finished = False
            while not action_finished:
                action_status_obj = self.connection.http_get(url=action_result_url)
                if action_status_obj.state == 'finished':
                    if action_status_obj.status == 'Successful':
                        action_finished = True
                        self._ixload = self._ixload + '/'
                        self.logger.info("Connected to L47, Session ID:%s" % (self._ixload))
                        msg = (
                                "Successfully connected to Ixload"
                                )
                        #self.add_error(msg)
                        self.warning(msg)   
                    else:
                        error_msg = "Error while executing action '%s'." % start_session_url
                        if action_status_obj.status == 'Error':
                            error_msg += action_status_obj.error
                        raise Exception(error_msg)
                else:
                    time.sleep(0.1)
            
        
    def _connect(self):
        """Connect to an IxNetwork API Server.
        """
        self._errors = []
        self.logger = setup_ixload_logger(self.log_level, module_name=__name__)
        self.logger.info("Connecting to L47")
        if self._assistant is None:
            self.sessionAssistance()

    def _request(self, method, url, payload=None, params=None, headers=None, option=None):
        '''
        '''
        data = json.dumps(payload)
        response = self.connection._request(method=method, url= url, data=data, params=params, headers=headers, option=option)
        return response
    
    def _wait_for_action_to_finish(self, reply_obj, action_url):
        '''
            This method waits for an action to finish executing. after a POST request is sent in order to start an action,
            The HTTP reply will contain, in the header, a 'location' field, that contains an URL.
            The action URL contains the status of the action. we perform a GET on that URL every 0.5 seconds until the action finishes with a success.
            If the action fails, we will throw an error and print the action's error message.
            Args:
            - reply_obj the reply object holding the location
            - rt_handle - the url pointing to the operation
        '''
        action_result_url = reply_obj.headers.get('location')
        if action_result_url:
            action_result_url = self.strip_api_and_version_from_url(action_result_url)
            action_finished = False
            while not action_finished:
                action_status_obj = self._request('GET', action_result_url)
                #action_status_obj = rt_handle.invoke('http_get', url=action_result_url)
                if action_status_obj.state == 'finished':
                    if action_status_obj.status == 'Successful':
                        action_finished = True
                    else:
                        error_msg = "Error while executing action '%s'." % action_url
                        if action_status_obj.status == 'Error':
                            error_msg += action_status_obj.error
                        raise Exception(error_msg)
                else:
                    time.sleep(0.1)
    
    def strip_api_and_version_from_url(self, url):
        '''
        #remove the slash (if any) at the beginning of the url
        '''
        if url[0] == '/':
            url = url[1:]
        url_elements = url.split('/')
        if 'api' in url:
            #strip the api/v0 part of the url
            url_elements = url_elements[2:]
        return '/'.join(url_elements)

    def _get_url(self, parent, child):
        '''
        '''
        try:
            pattern = re.search(r'(.*)(\{id\})(.*)', child)
            if pattern:
                url1 = parent + pattern.group(1)
                url2 = pattern.group(3)
                return (url1, url2)
        except Exception:
            raise Exception('can not get url for parent %s and child %s' % (parent, child))


    def _set_payload(self, snappi_obj, attr_map):
        properties = snappi_obj._properties
        payload = {}
        for snappi_attr, ixl_map in attr_map.items():
            value = snappi_obj.get(snappi_attr)
            payload[ixl_map] = value

        # for prop_name in properties:

        #     if prop_name != 'url' and isinstance(properties[prop_name], (str, int, float, list)):
        #         prop_name_new = self._convert_camel(prop_name)
        #         payload[prop_name_new] = properties[prop_name]
        return payload

    def _convert_camel(self, argument):
        word_regex_pattern = re.compile("[^A-Za-z]+")
        words = word_regex_pattern.split(argument)
        return "".join(w.lower() if i == 0 else w.title() for i, w in enumerate(words))

    def info(self, message):
        print ("working %s" % message)
        

    def warning(self, message):
        logging.warning(message)



import json
import re
from timer import Timer


class network(object):
    """Transforms OpenAPI objects into IxLoad objects
    - network to /network
    Args
    ----
    - ixloadapi (Api): instance of the Api class
    """
    
    def __init__(self, ixloadapi):
        self._api = ixloadapi
    
    def config(self):
        """Transform config.network into IxLoad.network
        1) create a network for every config.scenarios.networks that is not present in IxLoad
        2) set config.networks[].name to /network -name using rest call
        """
        self._scenarios = self._api.snappi_config.scenarios
        with Timer(self._api, "Network Configuration"):
            for scenario in self._scenarios:
                self._create_network(scenario)
    
    def _create_network(self, scenario):
        """Add any network to the api server that do not already exist
        """
        for network in scenario.networks:
            url = network.url
            scenario_url = scenario.url
            network_url = scenario_url + '/' + url
            network.url = network_url
            payload = self._api._set_payload(network)
            if payload:
                self._api._request('PATCH', network_url, payload)



from flask import Flask, request, jsonify, Response
#from exceptions import Snappil47Exception
import sys
#sys.path.insert(0, "c:\\Users\\waseebai\\Documents\\project\\snappi_l47\\snappi\\artifacts\\snappi")
#sys.path.insert(0, "C:\\Users\\waseebai\\Documents\\project\\GitHub\\snappi\\artifacts\\snappi")

import snappi

app = Flask(__name__)

global CONFIG 
CONFIG = snappi.Config()
CS = snappi.ControlState()
GM = snappi.MetricsRequest()
#sys.path.append("C:\\Users\\waseebai\\Documents\\project\\snappi_l47\\snappi-ixload")
from snappi_ixload import ixloadapi
API = ixloadapi.Api(host="localhost", version="10.00.0.152")



@app.route("/config", methods=["POST"])
def set_config():
    #print("server ----> ", request.data)
    CONFIG.deserialize(request.data.decode("utf-8"))
    try:
        response = API.set_config(CONFIG)
        warn = snappi.Warning()
        warn.warnings = ["Successfully configured IxLoad"]
        return Response(warn.serialize(), mimetype="application/json", status=200)
    except Exception as err:
        error = snappi.Error()
        error.code = err._status_code
        error.kind = "validation"
        error.errors = [err._message]
        print(err)
        return Response(
            status=400,
            response=error.serialize(),
            headers={"Content-Type": "application/json"},
        )
        return Response(err.status_code)
        #return Response(response=err.message, mimetype="application/json", status=err.status_code)
    # try:
    #     response = API.set_config(CONFIG)
    # except Exception as error:
    #     print("error")
    
    # warn = snappi.Warning()
    # warn.warnings = ["Successfully configured IxLoad"]
    # return Response(warn.serialize(), mimetype="application/json", status=200)

@app.route('/config', methods=['GET'])
def get_config():
    try:
        config = API.get_config()
        return Response(config.serialize(),
                    mimetype='application/json',
                    status=200)

    except Exception as error:
        print("error")
    
@app.route('/control/state', methods=["POST"])
def set_control_state():
    CS.deserialize(request.data.decode("utf-8"))
    
    try:
        
        config = API.set_control_state(CS)
        return Response(config.serialize(), mimetype="application/json", status=200)
        #return Response(config.serialize(),
        #            mimetype='application/json',
        #            status=200)
    except Exception as err:
        error = snappi.Error()
        error.code = err._status_code
        error.kind = "validation"
        error.errors = [err._message]
        print(err)
        return Response(
            status=400,
            response=error.serialize(),
            headers={"Content-Type": "application/json"},
        )
   


@app.route('/monitor/metrics', methods=["POST"])
def get_metrics():
    GM.deserialize(request.data.decode("utf-8"))
    
    try:
        
        config = API.metrics_request(CS)
        return Response(config.serialize(),
                    mimetype='application/json',
                    status=200)
    except Exception as error:
        print(error)

# main driver function
if __name__ == "__main__":
    app.run()
    #run_server()

#from snappi_ixload import l47server as x

#x.run_server()





import json
import re
from timer import Timer


class ethernet(object):
    """Transforms OpenAPI objects into IxNetwork objects
    - Lag to /lag
    Args
    ----
    - ixnetworkapi (Api): instance of the Api class
    """
    
    def __init__(self, ixloadapi):
        self._api = ixloadapi
    
    def config(self):
        """Transform config.ports into Ixnetwork.Vport
        1) delete any vport that is not part of the config
        2) create a vport for every config.ports[] that is not present in IxNetwork
        3) set config.ports[].location to /vport -location using resourcemanager
        4) set /vport/l1Config/... properties using the corrected /vport -type
        5) connectPorts to use new l1Config settings and clearownership
        """
        self._scenarios = self._api.snappi_config.scenarios
        with Timer(self._api, "Ethernet Configuration"):
            for scenario in self._scenarios:
                network = scenario.networks[-1]
                self._delete_ethernet(network)
                self._create_ethernet(network)
          
    
    def _create_ethernet(self, network):
        """Add any scenarios to the api server that do not already exist
        """
        for ethernet in network.ethernet:
            url1, url2 = self._api._get_url(network.url , ethernet.url)
            payload = {"itemType": "L2EthernetPlugin"}
            response = self._api._request('POST', url1, payload)
            ethernet.url = url1 + response + url2
            if network.ipv4:
                self._create_ipv4(network, ethernet.url)
            

    def _delete_ethernet(self, network):
        """delete any scenarios to the api server that do not already exist
        """
        for ethernet in network.ethernet:
            url1, url2 = self._api._get_url(network.url , ethernet.url)
            response = self._api._request('GET', url1, option=1)
            payload = {'objectID':response[0]['objectID']}
            response = self._api._request('DELETE', url1, payload)


    def _create_ipv4(self, network, ethernet_url):
        """
            Add any ipv4 to the api server that do not already exist
        """
        for ipv4 in network.ipv4:
            ipv4_url = ipv4.url
            eth_url = ethernet_url.replace('macRangeList', "")
            url1, url2 = self._api._get_url(eth_url, ipv4_url)
            payload = {"itemType": "IpV4V6Plugin"}
            response = self._api._request('POST', url1, payload)
            ipv4.url = url1 + response + url2
            payload = self._api._set_payload(ipv4)
            if payload:
                self._api._request('PATCH', ipv4.url, payload)
            return


import json
import re
import time
from snappi_ixload.timer import Timer

class port(object):
    """
    Args
    ----
    - Ixloadapi (Api): instance of the Api class

    """
        
    def __init__(self, ixloadapi):
        self._api = ixloadapi
    
    def config(self):
        """T
        """
        self._config = self._api._l47config
        with Timer(self._api, "Port Configuration"):
            self._create_chassis()
    
    def _create_chassis(self):
        """Add any scenarios to the api server that do not already exist
        """
        for device in self._config.devices:
            ethernet = device.ethernets[0]
            location = self._get_chasiss(ethernet.connection.port_name, self._config.ports)
            self._add_chassis(location)
            for ip in ethernet.ipv4_addresses:
                self._assign_ports(location, ip.name)
    
    def _get_chasiss(self, port_name, port_config):
        '''
        '''
        for port in port_config:
            if port.name == port_name:
                return port.location
            
    def _add_chassis(self, location):
        '''
        '''
        #
        chassis_name = location.split("/")[0]
        chassis_list_url = "%s/ixload/chassisChain/chassisList" % (self._api._ixload)
        payload = {"name": chassis_name}
        response = self._api._request('POST', chassis_list_url, payload)
        refresh_connection_url = "%s/%s/operations/refreshConnection" % (chassis_list_url, response)
        response = self._api._request('POST', refresh_connection_url, {})
        time.sleep(10)
        #self._api._wait_for_action_to_finish(response, refresh_connection_url)
    
    def _assign_ports(self, location, ip_name):
        '''
        
        '''
        active_test_url = "%s/ixload/test/activeTest" % (self._api._ixload)
        payload = {"enableForceOwnership": "true"}
        self._api._request('PATCH', active_test_url, payload)
        url = self._api._config_url.get(ip_name)
        url = self._api.common.get_community_url(url)
        port_list_url = url + "network/portList"
        chassis_id = 1
        chassis_ip, card_id, port_id = location.split("/")
        payload = {"chassisId": chassis_id, "cardId": card_id, "portId": port_id}
        #import pdb;pdb.set_trace()
        self._api._request('POST', port_list_url, payload)
            
            


import json
import re
import time
from snappi_ixload.timer import Timer

class interfaces(object):
    """Transforms OpenAPI objects into IxNetwork objects
    - Lag to /lag
    Args
    ----
    - Ixloadapi (Api): instance of the Api class

    """
    _ETHERNET = {
        "mac": "mac",
        "mtu": "mtu",
        "step": "incrementBy",
        #"count": "count"
    }

    _IP = {
        "address": "ipAddress",
        "gateway": "gatewayAddress",
        "prefix": "prefix",
        "name" : "name",
        "step": "incrementBy",
        "count": "count",
    }

    _VLAN = {
        "id": "firstId",
        "priority": "priority",
        "tpid": "tpid",
        "name" : "name"
    }
        
    def __init__(self, ixloadapi):
        self._api = ixloadapi
    
    def config(self):
        """T
        """
        self._devices_config = self._api._l47config.devices
        with Timer(self._api, "Interface Configuration"):
            self._create_devices()
    
    def _create_devices(self):
        """Add any scenarios to the api server that do not already exist
        """
        for device in self._devices_config:
            url1 = self._api._ixload + "ixload/test/activeTest/communityList"
            payload = {}
            response = self._api._request('POST', url1, payload)
            new_url = url1 + "/" + response + "/network/stack/childrenList"
            self._api._config_url[device.name] = new_url
            #self._delete_ethernet(device, new_url)
        for device in self._devices_config:
            self._create_ethernet(device)
            #self._create_ipv4()
            
  
    def _create_ethernet(self, device):       
        """Add any scenarios to the api server that do not already exist
        """
        flag = 1
        for ethernet in device.ethernets:
            if flag:
                #payload = {"itemType": "L2EthernetPlugin"}
                url = self._api._config_url[device.name]
                response = self._api._request('GET', url)
                self._api._config_url["mac_url"] = url + "/" + str(response[-1]['objectID'])
                eth_url = url + "/" + str(response[-1]['objectID']) + "/macRangeList/"
                ipp_url = url + "/" + str(response[-1]['objectID']) + "/childrenList/"
                mac_url =  eth_url + str(self._api._request('GET', eth_url)[-1]['objectID'])
                ip_url =  ipp_url + str(self._api._request('GET', ipp_url)[-1]['objectID']) + "/rangeList/"
                ip_url = ip_url + str(self._api._request('GET', ip_url)[-1]['objectID'])
                
                vlan_url = url + "/" + str(response[-1]['objectID']) + "/vlanRangeList/"
                vlan_url =  vlan_url + str(self._api._request('GET', vlan_url)[-1]['objectID'])
                payload = {"autoMacGeneration": False}
                response = self._api._request('PATCH', ip_url, payload)
                payload = self._api._set_payload(ethernet, interfaces._ETHERNET)
                response = self._api._request('PATCH', mac_url, payload)
                self._api._config_url[ethernet.name] = mac_url
                self._create_ipv4(ethernet, ip_url, flag)
                self._create_vlan(ethernet, vlan_url, flag)
                flag = 0
            else:
                eth_url = self._api._config_url["mac_url"] + "/macRangeList"
                payload = self._api._set_payload(ethernet, interfaces._ETHERNET)
                response = self._api._request('POST', eth_url, payload)
                eth_url = eth_url + "/" + str(response)
                ip_url = self._api._config_url["mac_url"] + "/childrenList/"
                ip_url = ip_url + str(self._api._request('GET', ip_url, option=1)[-1]['objectID']) + "/rangeList/"
                ip_url = ip_url + str(self._api._request('GET', ip_url, option=1)[-1]['objectID'])
                
                vlan_url = self._api._config_url["mac_url"] + "/vlanRangeList/"
                vlan_url = vlan_url + str(self._api._request('GET', vlan_url, option=1)[-1]['objectID'])
                payload = {"autoMacGeneration": False}
                response = self._api._request('PATCH', ip_url, payload)
                self._api._config_url[ethernet.name] = eth_url
                self._create_ipv4(ethernet, ip_url, flag)
                self._create_vlan(ethernet, vlan_url, flag)
            

    def _delete_ethernet(self, device, url):
        """delete any scenarios to the api server that do not already exist
        """
        response = self._api._request('GET', url, option=1)
        payload = {'objectID':response[0]['objectID']}
        response = self._api._request('DELETE', url, payload)


    def _create_ipv4(self, ethernet, url, flag):
        """
            Add any ipv4 to the api server that do not already exist
        """
        ipv4_addresses = ethernet.get("ipv4_addresses")
        if ipv4_addresses is None:
            return
        for ipv4 in ethernet.ipv4_addresses: 
            payload = self._api._set_payload(ipv4, interfaces._IP)
            if payload:
                response = self._api._request('PATCH', url, payload)
                self._api._config_url[ipv4.name] = url
    
    def _create_vlan(self, ethernet, vlan_url, flag):
        """
            Add any ipv4 to the api server that do not already exist
        """
        
        vlans = ethernet.get("vlans")
        if vlans is None:
            return
        for vlan in ethernet.vlans:
            payload = {"enabled": True}
            response = self._api._request('PATCH', vlan_url, payload)
            payload = self._api._set_payload(vlan, interfaces._VLAN)
            if payload:
                response = self._api._request('PATCH', vlan_url, payload)
                self._api._config_url[vlan.name] = vlan_url



import sys
import logging as log
import pytest
sys.path.append("C:\\Users\\waseebai\\Documents\\GitHub\\snappi\\artifacts\\snappi")
import snappi
import time
import datetime
#from table import table

def test_snappi(request):

    api = snappi.api(location="https://10.39.71.237:443", ext='ixnetwork')
    # new config
    config = api.config()
    # port location is chassis-ip;card-id;port-id
    tx, rx = (
        config.ports
        .port(name='p1', location="10.39.64.169;2;1")
        .port(name='p2', location="10.39.64.169;2;2")
    )
    config.options.port_options.location_preemption = True
    # configure layer 1 properties
    ly, = config.layer1.layer1(name='ly')
    ly.port_names = [tx.name, rx.name]
    ly.speed = ly.SPEED_10_GBPS
    ly.media = ly.FIBER
    # configure flow properties
    flw, = config.flows.flow(name='flw')
    # flow endpoints
    flw.tx_rx.port.tx_name = tx.name
    flw.tx_rx.port.rx_name = rx.name
    # enable flow metrics
    flw.metrics.enable = True
    # configure rate, size, frame count
    flw.size.fixed = 128
    flw.rate.pps = 1000
    flw.duration.fixed_packets.packets = 10000
    # configure protocol headers with defaults fields
    flw.packet.ethernet().vlan().ipv4().tcp()
    # push configuration
    api.set_config(config)
    print("L23 config completed")

def test_snappi_load():
    print("Working on L47 Config")
    api = snappi.api(location="http://127.0.0.1:5000", verify=False)

    config = api.config()

    port_1 = config.ports.port(name="p1", location="10.39.46.4/11/3")[-1]
    port_2 = config.ports.port(name="p2", location="10.39.46.4/11/7")[-1]

    (d1, d2) = config.devices.device(name="d1").device(name="d2")
    (e1,) = d1.ethernets.ethernet(name="d1.e1")
    e1.connection.port_name = "p1"
    e1.mac = "00:00:01:00:00:01"
    (e2,) = d2.ethernets.ethernet(name="d2.e2")
    e2.connection.port_name = "p2"
    e2.mac = "00:00:02:00:00:01"
    (ip1,) = e1.ipv4_addresses.ipv4(name="e1.ipv4")
    ip1.address = "173.173.173.10"
    ip1.gateway = "0.0.0.0"
    (ip2,) = e2.ipv4_addresses.ipv4(name="e2.ipv4")
    ip2.address = "173.173.173.20"
    ip2.gateway = "0.0.0.0"

    (a1, a2) = config.apps.app(name="a1").app(name="a2")
    (t1,) = a1.tcp.tcp(name="tcp1")
    t1.keep_alive_time = 120
    (t2,) = a2.tcp.tcp(name="tcp2")
    t2.keep_alive_time = 120

    (h1,) = a1.http_client.httpclient()
    h1.client.name="HTTP_client1"
    h1.client.transport_name = t1.name
    h1.client.command_timeout = 60
    h1.client.cookie_reject_probability = False
    (m1,) = h1.client.methods.get()
    m1.server = "server1"
    m1.page = "/64k.html"


    (h2,) = a2.http_server.httpserver()
    #h2.server.name="HTTP_server1"
    h2.server.name = "HTTP_server1"
    h2.server.transport_name = t2.name
    h2.server.http_port = 80

    #Traffic configs

    traffic = a1.stateful_flows.statefulflow()[-1]
    traffic.client.protocol = h1.client.name
    traffic_c_ep=traffic.client.endpoints.client()[-1]
    traffic_c_ep.ip_interface_name = ip1.name

    traffic_c_ep.ports.values = [1]

    traffic.server.protocol = h2.server.name  
    traffic_sl_ep = traffic.server.endpoints.server()[-1]
    traffic_sl_ep.dest.name = ip2.name
    traffic_sl_ep.ports.values = [1]
    response = api.set_config(config)
    print(response)
    print("********************************")
    response = api.get_config()
    print(response)
    print("********************************")



if __name__ == "__main__":
    pytest.main(["-s", __file__])


'''
    DESCRIPTION
    IxRestUtils is a collection of classes that offer a generic wrapper around a raw REST API.
    It handles:
    - Creating a connection;
    - Running HTTP methods for an active connection
    Abstracting the RAW HTTP input / output to tangible objects that will act as an interface to the REST API.
'''
import requests
from urllib.parse import urljoin
import json
import re
try:
    requests.packages.urllib3.disable_warnings()
except Exception:
    pass
# Gets a Connection instance, that will be used to make the HTTP requests to the application
def get_connection(server, port, http_redirect=False):
    '''
        getconnection perform connection to Ixload and returns session Url.
    '''
    transport_type = 'https' if not http_redirect else 'http'
    connection_url = "%s://%s:%s/" % (transport_type, server, port)
    conn = Connection(connection_url, "v0", http_redirect)
    return conn

class Connection(object):
    '''
        Class that executes the HTTP requests to the application instance.
        It handles creating the HTTP session and executing HTTP methods.

    '''
    kHeaderContentType = "content-type"
    kContentJson = "application/json"
    kApiKeyHeader = 'X-Api-Key'
    kApiKey = ''

    def __init__(self, siteUrl, apiVersion, http_redirect=False):
        '''
            Args:
            - siteUrl is the actual url to which the Connection instance will be made.
            - apiVersion is the actual version of the REST API that the Connection instance will use.
            The HTTP session will be created when the first http request is made.
        '''
        self.http_session = None
        self.http_redirect = http_redirect
        self.url = Connection.urljoin(siteUrl, "api")
        self.url = Connection.urljoin(self.url, apiVersion)

    @staticmethod
    def set_api_key(api_key):
        '''
            set_api_key will set connection key for REST calls.
        '''
        Connection.kApiKey = api_key

    def _get_http_session(self):
        '''
            This is a lazy initializer for the HTTP session.
            It does not need to be active until it is required.
        '''
        if self.http_session is None:
            self.http_session = requests.Session()
            if not self.http_redirect:
                from requests.adapters import HTTPAdapter
                from requests.packages.urllib3.poolmanager import PoolManager
                import ssl
                http_adapter = HTTPAdapter()
                http_adapter.poolmanager = PoolManager(ssl_version=ssl.PROTOCOL_TLSv1)
                self.http_session.mount('https://', http_adapter)
        return self.http_session

    @classmethod
    def urljoin(cls, base, end):
        """ Join two URLs. If the second URL is absolute, the base is ignored.
        Use this instead of urlparse.urljoin directly so that we can customize its behavior if necessary.
        Currently differs in that it
            1. appends a / to base if not present.
            2. casts end to a str as a convenience
        """
        if base and not base.endswith("/"):
            base = base + "/"
        return urljoin(base, str(end))
    
    def _request(self, method, url="", data="", option = None, params=None, headers=None):
        '''
        '''
        if method == "POST":
            reply = self.http_post(url, data, params, headers)
            if not reply.ok:
                raise Exception(reply.text)
            try:
                new_obj_path = reply.headers['location']
            except:
                raise Exception("Location header is not present. Please check if the action was created successfully.")
            if option:
                return reply
            response = new_obj_path.split('/')[-1]
            return response
        elif method == "GET":
            response = self.http_get(url=url , option = option)
            return response
        elif method == "PATCH":
            reply = self.http_patch(url, data, params, headers)
            if not reply.ok:
                raise Exception(reply.text)
            return reply
        elif method == "DELETE":
            reply = self.http_delete(url, data, params, headers)
            if not reply.ok:
                raise Exception(reply.text)
            return reply

    def http_request(self, method, url="", data="", params=None, headers=None):
        '''
            Args:

            - Method (mandatory) represents the HTTP method that will be executed.
            - url (optional) is the url that will be appended to the application url.
            - data (optional) is the data that needs to be sent along with the HTTP method as the JSON payload
            - params (optional) the payload python dict not necessary if data is used.
            - headers (optional) these are the HTTP headers that will be sent along with the request. If left blank will use default

            Method for making a HTTP request. The method type (GET, POST, PATCH, DELETE) will be sent as a parameter.
            Along with the url and request data. The HTTP response is returned
        '''
        params = {} if params is None else params
        headers = {} if headers is None else headers
        headers[Connection.kHeaderContentType] = Connection.kContentJson
        if Connection.kApiKey != '':
            headers[Connection.kApiKeyHeader] = Connection.kApiKey
        if isinstance(data, dict):
            data = json.dumps(data)
        abs_url = Connection.urljoin(self.url, url)
        result = self._get_http_session().request(method, abs_url, data=str(data), params=params, headers=headers, verify=False)
        return result

    def http_get(self, url="", data="", option = None, params=None, headers=None, error_codes=None):
        '''
            Method for calling HTTP GET. This will return a WebObject that has the fields returned
            in JSON format by the GET operation.
        '''
        params = {} if params is None else params
        headers = {} if headers is None else headers
        error_codes = [] if error_codes is None else error_codes
        reply = self.http_request("GET", url, data, params, headers)
        if reply.status_code in error_codes:
            raise Exception("Error on executing GET request on url %s: %s" % (url, reply.text))
        if option:
            return reply.json()
        else:
            return _format_response(reply.json(), url)

    def http_options(self, url="", data="", params=None, headers=None, error_codes=None):
        '''
            Method for calling HTTP Option. This will return a WebObject that has the fields returned
            in JSON format by the Option operation.
        '''
        params = {} if params is None else params
        headers = {} if headers is None else headers
        error_codes = [] if error_codes is None else error_codes
        reply = self.http_request("OPTIONS", url, data, params, headers)
        if reply.status_code in error_codes:
            raise Exception("Error on executing Options request on url %s: %s" % (url, reply.text))
        return _format_response(reply.json(), url)

    def http_post(self, url="", data="", params=None, headers=None):
        '''
            Method for calling HTTP POST. Will return the HTTP reply.
        '''
        params = {} if params is None else params
        headers = {} if headers is None else headers
        return self.http_request("POST", url, data, params, headers)

    def http_patch(self, url="", data="", params=None, headers=None):
        '''
            Method for calling HTTP PATCH. Will return the HTTP reply.
        '''
        params = {} if params is None else params
        headers = {} if headers is None else headers
        return self.http_request("PATCH", url, data, params, headers)

    def http_delete(self, url="", data="", params=None, headers=None):
        '''
            Method for calling HTTP DELETE. Will return the HTTP reply.
        '''
        params = {} if params is None else params
        headers = {} if headers is None else headers
        return self.http_request("DELETE", url, data, params, headers)

def _format_response(value, _url_=None):
    '''
        Method used for creating a wrapper object corresponding to the JSON string received on a GET request.
    '''
    if isinstance(value, dict):
        if _url_ and bool(re.search(r'[\w.-]+/ixload/stats/[\w().-]+/values', _url_)):
            pass#'values' resources only have name and value of the stats, don't need the url
        else:
            value['_url_'] = _url_
        result = WebObject(**value)
        #result = value
    elif isinstance(value, list):
        #result = WebList(entries=value, _url_=_url_)
        result = value
    else:
        result = value
    return result

class WebList(list):
    '''
        Using this class a JSON list will be transformed in a list of WebObject instances.
    '''
    def __init__(self, entries=None, _url_=None):
        '''
            Create a WebList from a list of items that are processed by the _format_response function
        '''
        list.__init__(self)
        entries = [] if entries is None else entries
        self._url_ = _url_
        url = _url_
        filter_syntax = "?filter=" # we need to remove the query param syntax from all chindren of the list.
        if url and filter_syntax in url:
            url = url.split(filter_syntax)[0] # get everything on the left of the filter, removing the query param
        for item in entries:
            item_url = None
            if "objectID" in item:
                item_url = "%s/%s" % (url, item["objectID"])
            self.append(_format_response(item, item_url))

    def copy_data(self, new_obj):
        '''
            copy date and retrun json
        '''
        self[:] = []
        for item in new_obj:
            self.append(item)


class WebObject(object):
    '''
        A WebObject instance will have its fields set to correspond to the JSON format received on a GET request.
        for example: a response in the format: {"caption": "http"} will return an object that has obj.caption="http"
    '''
    def __init__(self, **entries):
        '''
            Create a WebObject instance by providing a dict having a property - value structure.
        '''
        self.json_options = {}
        for key, value in entries.items():
            web_obj = _format_response(value)
            self.json_options[key] = web_obj
            self.__dict__[key] = web_obj

    def copy_data(self, new_obj):
        '''
            copy date and retrun json
        '''
        self.json_options = {}
        for key, obj in new_obj.json_options.items():
            self.json_options[key] = obj
            self.__dict__[key] = obj

    def get_options(self):
        '''
            Get the JSON dictionary which represents the WebObject Instance
        '''
        return self.json_options



import json
import re
import time
from brisk_ixload.timer import Timer

class stats(object):
    """Transforms OpenAPI objects into IxNetwork objects
    - Lag to /lag
    Args
    ----
    - ixnetworkapi (Api): instance of the Api class
    """
    
    def __init__(self, ixloadapi):
        self._api = ixloadapi
    
    def config(self):
        """Transform config.ports into Ixnetwork.Vport
        1) delete any vport that is not part of the config
        2) create a vport for every config.ports[] that is not present in IxNetwork
        3) set config.ports[].location to /vport -location using resourcemanager
        4) set /vport/l1Config/... properties using the corrected /vport -type
        5) connectPorts to use new l1Config settings and clearownership
        """
        self._scenarios_config = self._api.brisk_config
        

        with Timer(self._api, "Scenario Configuration"):
            self._create_scenarios()
    
    def _create_scenarios(self):
        """Add any scenarios to the api server that do not already exist
        """
        for scenario in self._scenarios_config:
            url1, url2 = self._api._get_url(self._api._ixload , scenario.url)
            payload = {}
            response = self._api._request('POST', url1, payload)
            new_url = url1 + response + url2
            scenario.url = new_url
            self._create_network(scenario, scenario.url)
            



import json
import re
from timer import Timer


class ipv4(object):
    """Transforms OpenAPI objects into IxLoad objects
    - ipv4 to /ipv4
    Args
    ----
    - ixloadapi (Api): instance of the Api class
    """
    
    def __init__(self, ixloadapi):
        self._api = ixloadapi
    
    
    def config(self):
        """Transform config.network into IxLoad.network
        1) create a network for every config.scenarios.networks that is not present in IxLoad
        2) set config.networks[].name to /network -name using rest call
        """
        self._scenarios = self._api.snappi_config.scenarios
        with Timer(self._api, "Ipv4 Configuration"):
            for scenario in self._scenarios:
                network = scenario.networks[-1]
                self._create_ipv4(network)
    

    def _delete_ipv4(self, network):
        for ipv4 in network.ipv4:
            ipv4_url = ipv4.url
            ethernet_url = network.ethernet.url.replace('macRangeList', "")
            url1, url2 = self._api._get_url(ethernet_url, ipv4_url)
            response = self._api._request('GET', url1, option=1)
            payload = {'objectID':response[0]['objectID']}
            self._api._request('DELETE', url1, payload)
          

    def _create_ipv4(self, network):
        """
            Add any ipv4 to the api server that do not already exist
        """
        for ipv4 in network.ipv4:
            ipv4_url = ipv4.url
            print(network)
            ethernet_url = ethernet_url.replace('macRangeList', "")
            import pdb; pdb.set_trace()
            url1, url2 = self._api._get_url(ethernet_url, ipv4_url)
            payload = {"itemType": "IpV4V6Plugin"}
            response = self._api._request('POST', url1, payload)
            ipv4.url = url1 + response + url2
            payload = self._api._set_payload(ipv4)
            if payload:
                self._api._request('PATCH', ipv4.url, payload)


import json
import re
import time
from brisk_ixload.timer import Timer

class scenarios(object):
    """Transforms OpenAPI objects into IxNetwork objects
    - Lag to /lag
    Args
    ----
    - ixnetworkapi (Api): instance of the Api class
    """
    
    def __init__(self, ixloadapi):
        self._api = ixloadapi
    
    def config(self):
        """Transform config.ports into Ixnetwork.Vport
        1) delete any vport that is not part of the config
        2) create a vport for every config.ports[] that is not present in IxNetwork
        3) set config.ports[].location to /vport -location using resourcemanager
        4) set /vport/l1Config/... properties using the corrected /vport -type
        5) connectPorts to use new l1Config settings and clearownership
        """
        self._scenarios_config = self._api.brisk_config.scenarios
        with Timer(self._api, "Scenario Configuration"):
            self._create_scenarios()
    
    def _create_scenarios(self):
        """Add any scenarios to the api server that do not already exist
        """
        for scenario in self._scenarios_config:
            url1, url2 = self._api._get_url(self._api._ixload , scenario.url)
            payload = {}
            response = self._api._request('POST', url1, payload)
            new_url = url1 + response + url2
            scenario.url = new_url
            self._create_network(scenario, scenario.url)
            
    def _create_network(self, scenario, scenario_url):
        """Add any network to the api server that do not already exist
        """
        for network in scenario.networks:
            url = network.url
            network.url = scenario_url + '/' + url
            payload = self._api._set_payload(network)
            if payload:
                self._api._request('PATCH', network.url, payload)
            if network.ethernet:
                self._create_ethernet(network, network.url)
            if network.ports:
                self._create_port(network, network.url)

    def _create_port(self, network, network_url):
        """Add any scenarios to the api server that do not already exist
        """
        for port in network.ports:
            url  = "%s/ixload/test/activeTest" % self._api._ixload
            payload = {"enableForceOwnership": "true"}
            self._api._request('PATCH', url, payload)
            #url1, url2 = self._api._get_url(network_url , ethernet.url)
            port.url = network_url + port.url
            payload = self._api._set_payload(port)
            #
            response = self._api._request('POST', port.url, payload)
            
    def _create_ethernet(self, network, network_url):
        """Add any scenarios to the api server that do not already exist
        """
        for ethernet in network.ethernet:
            url1, url2 = self._api._get_url(network_url , ethernet.url)
            payload = {"itemType": "L2EthernetPlugin"}
            response = self._api._request('POST', url1, payload)
            ethernet.url = url1 + response + url2
            if network.ipv4:
                self._create_ipv4(network, ethernet.url)
            

    def _delete_ethernet(self, network):
        """delete any scenarios to the api server that do not already exist
        """
        for ethernet in network.ethernet:
            url1, url2 = self._api._get_url(network.url , ethernet.url)
            response = self._api._request('GET', url1, option=1)
            payload = {'objectID':response[0]['objectID']}
            response = self._api._request('DELETE', url1, payload)


    def _create_ipv4(self, network, ethernet_url):
        """
            Add any ipv4 to the api server that do not already exist
        """
        for ipv4 in network.ipv4:
            ipv4_url = ipv4.url
            eth_url = ethernet_url.replace('macRangeList', "")
            url1, url2 = self._api._get_url(eth_url, ipv4_url)
            payload = {"itemType": "IpV4V6Plugin"}
            response = self._api._request('POST', url1, payload)
            ipv4.url = url1 + response + url2
            payload = self._api._set_payload(ipv4)
            if payload:
                self._api._request('PATCH', ipv4.url, payload)
            return



import time

class Timer:
    def __init__(self, api, msg):
        self._api = api
        self._msg = msg

    def __enter__(self):
        """Start a new timer as a context manager
        """
        self._start = time.time()
        return self

    def __exit__(self, *exc_info):
        """Stop the context manager timer
        """
        self._api.info(self._msg + ' %.3fs' % (time.time() - self._start))
    



import json
import re
import time
from timer import Timer
from common import Common

class tcp_config(Common):
    """Transforms OpenAPI objects into IxNetwork objects
    - Lag to /lag
    Args
    ----
    - Ixloadapi (Api): instance of the Api class

    """
    _HTTP_CLIENT = {
        "command_timeout": "commandTimeout"
    }

    _IP = {
        "address": "ipAddress",
        "gateway": "gatewayAddress",
        "prefix": "prefix",
        "name" : "name"
    }
    _TCP = {
        "keep_alive_time": "tcp_keepalive_time",
        "keep_alive_interval" : "tcp_keepalive_intvl",
        "receive_buffer_size" : "tcp_rmem_default",
        "transmit_buffer_size" : "tcp_wmem_default",
        "adjust_tcp_buffers" : "adjust_tcp_buffers",
        "retransmission_minimum_timeout" : "tcp_rto_min",
        "retransmission_maximum_timeout" : "tcp_rto_max",
        "minimum_source_port" : "tcp_port_min",
        "maximum_source_port" : "tcp_port_max",
        "inter_packet_gap" : "llm_hdr_gap",
        "inter_packet_delay" : "inter_packet_delay",
        "ip_fragment_time" : "tcp_ipfrag_time",
        "fin_timeout" : "tcp_fin_timeout",
        "syn_retries" : "tcp_syn_retries",
        "synack_retries" : "tcp_synack_retries",
        "retansmit_retries1" : "tcp_retries1",
        "retransmit_retries2" : "tcp_retries2",
        "packet_reordering" : "tcp_reordering",
        "delayed_acks_segments" : "delayed_acks_segments",
        "delayed_acks_timeout" : "delayed_acks_timeout",
        "port_randomization" : "udp_port_randomization",
        "disable_path_mtu" : "ip_no_pmtu_disc",
        "window_scaling" : "tcp_window_scaling",
        "selective_ack" : "tcp_sack",
        "time_wait_reuse": "tcp_tw_reuse" ,
        "time_wait_recycle" : "tcp_tw_recycle",
        "time_wait_rfc1323_strict" : "tcp_tw_rfc1323_strict",
        "packet_timestamps" : "tcp_timestamps",
        "explicit_congestion_notification" : "tcp_ecn",
        #"source_port" : "",
        #"destination_port" : "",
        "bic" : "tcp_bic",
        "vegas_alpha": "tcp_vegas_alpha",
        "rfc1337": "tcp_rfc1337",
        "mem_low": "tcp_mem_low",
        "maximum_wmem": "tcp_wmem_max",
        "westwood" : "tcp_westwood",
        "avoid_vegas_congestion" : "tcp_vegas_cong_avoid",
        "maximum_rmem" : "tcp_rmem_max",
        "orphan_retries" : "tcp_orphan_retries",
        "maximum_time_wait_buckets" : "tcp_max_tw_buckets",
        "low_latency" : "tcp_low_latency",
        "minimum_rmem" : "tcp_rmem_min",
        "window_scale" : "tcp_adv_win_scale",
        "wmem_default" : "tcp_wmem_default",
        "minimum_wmem" : "tcp_wmem_min",
        "stdurg" : "tcp_stdurg",
        "maximum_syn_backlog": "tcp_max_syn_backlog",
        "dsack" : "tcp_dsack",
        "abort_on_overflow": "tcp_abort_on_overflow",
        "fragment_reassembly_timer": "tcp_frto",
        "vegas_beta" : "tcp_vegas_beta",
        "maximum_orphans" : "tcp_max_orphans",
        "mem_pressure" : "tcp_mem_pressure",
        "moderate_receive_buffer" : "tcp_moderate_rcvbuf",
        "no_metrics_save" : "tcp_no_metrics_save",
        "retrans_collapse" : "tcp_retrans_collapse",
        "rmem_default" : "tcp_rmem_default",
        "mem_high": "tcp_mem_high",
        "vegas_gamma" : "tcp_vegas_gamma",
        "fack": "tcp_fack",
        #"tcp_bic_fast_convergence": "",
        "bic_low_window": "tcp_bic_low_window",
        "app_win": "tcp_app_win",
        "keep_alive_probes" : "tcp_keepalive_probes"
    }
        
    def __init__(self, ixloadapi):
        self._api = ixloadapi
    
    def config(self):
        """
        """
        self._devices_config = self._api._l47config.devices
        with Timer(self._api, "Tcp Configurations"):
            self._update_tcp()
    
    def _update_tcp(self):
        """Add any scenarios to the api server that do not already exist
        """
        
        for device in self._devices_config:
            #
            self._update_tcp_config(device)
            
    def _update_tcp_config(self, device):
        """Add any scenarios to the api server that do not already exist
        """
        #
        for tcp in device.tcps:
            url = self._api._config_url.get(tcp.ip_interface_name)
            #url = self._api._config_url.get(ip_object)
            url = self.get_community_url(url)
            tcp_child_url = "%snetwork/globalPlugins" % url
            response_list = self._api._request('GET', tcp_child_url)
            for index in range(len(response_list)):
                if response_list[index]['itemType'] == 'TCPPlugin':
                    tcp_url = "%s/%s" % (tcp_child_url, response_list[index]['objectID'])
                    self._api.logger.info("tcp_url:%s" % (tcp_url))
                    payload = self._api._set_payload(tcp, tcp_config._TCP)
                    response = self._api._request('PATCH', tcp_url, payload)
                    self._api._config_url[tcp.name] = tcp_url
            

    # def _delete_ethernet(self, network):
    #     """delete any scenarios to the api server that do not already exist
    #     """
    #     for ethernet in network.ethernet:
    #         url1, url2 = self._api._get_url(network.url , ethernet.url)
    #         response = self._api._request('GET', url1, option=1)
    #         payload = {'objectID':response[0]['objectID']}
    #         response = self._api._request('DELETE', url1, payload)
